package org.gnomekr.argosdoc.node;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.commons.lang.NullArgumentException;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

/**
 * ElementNode.java
 * @author Xavier Cho
 * @version $Revision: 1.2 $ $Date: 2005/11/18 09:44:50 $
 */
public class ElementNode implements ITreeNode<ElementNode> {

    static final long serialVersionUID = -8495611373423919782L;

    private String name;
    private NodeType type;
    private URL url;
    private ElementNode parent;
    private List<ElementNode> children = new ArrayList<ElementNode>();

    public ElementNode(String name, NodeType type) {
        setName(name);
        setType(type);
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#getName()
     */
    public String getName() {
        return name;
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#setName(java.lang.String)
     */
    public void setName(String name) {
        if (name == null) {
            throw new NullArgumentException("name");
        }
        this.name = name;
    }

    public URL getURL() {
        return url;
    }

    public void setURL(URL url) {
        this.url = url;
    }

    public NodeType getType() {
        return type;
    }

    public void setType(NodeType type) {
        if (type == null) {
            throw new NullArgumentException("type");
        }
        this.type = type;
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#getParent()
     */
    public ElementNode getParent() {
        return parent;
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#setParent(null)
     */
    public void setParent(ElementNode parent) {
        this.parent = parent;
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#findChild(java.lang.String)
     */
    public ElementNode findChild(String name) {
        ElementNode node = null;

        for (ElementNode child : children) {
            if (child.getName().equals(name)) {
                node = child;
                break;
            }

            if ((node = child.findChild(name))!= null) {
                break;
            }
        }

        return node;
    }

    public ElementNode findChild(URL url) {
        ElementNode node = null;

        for (ElementNode child : children) {
            if (url.equals(child.getURL())) {
                node = child;
                break;
            }

            if ((node = child.findChild(url))!= null) {
                break;
            }
        }

        return node;
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#addChildNode(null)
     * @param child
     */
    public void addChildNode(ElementNode child) {
        if (child == null) {
            throw new NullArgumentException("child");
        }

        if (!children.contains(child)) {
            children.add(child);
        }
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#getChildCount()
     */
    public int getChildCount() {
        return children.size();
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#getChildren()
     */
    public ElementNode[] getChildren() {
        return (ElementNode[]) children
                .toArray(new ElementNode[children.size()]);
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#isLeaf()
     */
    public boolean isLeaf() {
        return getChildCount() == 0;
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#removeChildNode(null)
     * @param child
     */
    public void removeChildNode(ElementNode child) {
        if (child == null) {
            throw new NullArgumentException("child");
        }

        if (children.contains(child)) {
            children.remove(child);
        }
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof ElementNode) {
            ElementNode node = (ElementNode) obj;

            EqualsBuilder builder = new EqualsBuilder();
            builder.append(getName(), node.getName());
            builder.append(getParent(), node.getParent());

            return builder.isEquals();
        }

        return super.equals(obj);
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        HashCodeBuilder builder = new HashCodeBuilder();
        builder.append(getName());
        builder.append(getParent());

        return builder.toHashCode();
    }

    /**
     * @see org.gnomekr.argosdoc.node.ITreeNode#sortChildren(java.util.Comparator)
     */
    public void sortChildren(Comparator<? super ElementNode> comparator) {
        if (comparator == null) {
            throw new NullArgumentException("comparator");
        }

        if (!isLeaf()) {
            Collections.sort(children, comparator);
        }

        for (ElementNode child : children) {
            child.sortChildren(comparator);
        }
    }
}
