package org.gnomekr.argosdoc.view;

import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;
import org.gnomekr.argosdoc.APIBrowserPlugin;
import org.gnomekr.argosdoc.node.DocumentNode;
import org.gnomekr.argosdoc.node.ElementNode;
import org.gnomekr.argosdoc.node.NodeComparator;
import org.gnomekr.argosdoc.node.NodeFactory;
import org.gnomekr.argosdoc.node.NodeType;

/**
 * DocumentsTreeView.java
 * @author Xavier Cho
 * @version $Revision: 1.5 $ $Date: 2005/11/18 09:44:50 $
 */
public class DocumentsTreeView extends ViewPart {

    public static final String ID = "argosdoc.view.DocumentsTreeView";
    public static final String DOCUMENT_PATHS = "document.paths";

    private IMemento memento;
    private TreeViewer viewer;
    private List<DocumentNode> documents = new ArrayList<DocumentNode>();

    /**
     * The content provider class is responsible for providing objects to the
     * view. It can wrap existing objects in adapters or simply return objects
     * as-is. These objects may be sensitive to the current input of the view,
     * or ignore it and always show the same content (like Task List, for
     * example).
     */
    class ViewContentProvider implements ITreeContentProvider {

        /**
         * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
         */
        public Object[] getChildren(Object parentElement) {
            if (parentElement instanceof ElementNode) {
                return ((ElementNode) parentElement).getChildren();
            }

            return null;
        }

        /**
         * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
         */
        public Object getParent(Object element) {
            if (element instanceof ElementNode) {
                return ((ElementNode) element).getParent();
            }

            return null;
        }

        /**
         * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
         */
        public boolean hasChildren(Object element) {
            if (element instanceof ElementNode) {
                return !((ElementNode) element).isLeaf();
            }

            return false;
        }

        /**
         * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
         */
        @SuppressWarnings("unchecked")
        public Object[] getElements(Object inputElement) {
            if (inputElement instanceof List) {
                List elements = (List) inputElement;
                return elements.toArray(new DocumentNode[elements.size()]);
            }

            return null;
        }

        /**
         * @see org.eclipse.jface.viewers.IContentProvider#dispose()
         */
        public void dispose() {
        }

        /**
         * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
         */
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
        }
    }

    class ViewLabelProvider extends LabelProvider {

        /**
         * @see org.eclipse.jface.viewers.ILabelProvider#getImage(java.lang.Object)
         */
        @Override
        public Image getImage(Object element) {
            ImageRegistry registry = APIBrowserPlugin.getDefault()
                    .getImageRegistry();

            Image image = null;
            if (element instanceof ElementNode) {
                NodeType type = ((ElementNode) element).getType();
                String iconPath = type.getIconPath();
                if (iconPath != null) {
                    image = registry.get(iconPath);

                    if (image == null) {
                        ImageDescriptor descriptor = APIBrowserPlugin
                                .getImageDescriptor(iconPath);
                        image = descriptor.createImage();

                        registry.put(iconPath, image);
                    }
                }
            }

            if (image == null) {
                image = super.getImage(element);
            }

            return image;
        }

        /**
         * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
         */
        @Override
        public String getText(Object element) {
            if (element instanceof ElementNode) {
                return ((ElementNode) element).getName();
            }

            return super.getText(element);
        }
    }

    /**
     * This is a callback that will allow us to create the viewer and initialize
     * it.
     */
    public void createPartControl(Composite parent) {
        viewer = new TreeViewer(parent, SWT.SINGLE | SWT.H_SCROLL
                | SWT.V_SCROLL);
        viewer.setContentProvider(new ViewContentProvider());
        viewer.setLabelProvider(new ViewLabelProvider());

        IViewSite site =  getViewSite();
        site.setSelectionProvider(viewer);
        site.getPage().addSelectionListener(new ISelectionListener() {

            public void selectionChanged(
                    IWorkbenchPart part,
                    ISelection selection) {
                if (selection instanceof IStructuredSelection) {
                    Object value = ((IStructuredSelection) selection)
                            .getFirstElement();
                    if (value instanceof URL) {
                        selectNodeWithURL((URL) value);
                    }
                }
            }
        });

        if (memento != null) {
            final String paths = memento.getString(DOCUMENT_PATHS);

            if (paths != null) {
                final String title = "Parsing JavaDoc document...";

                IWorkbenchWindow window = getViewSite().getWorkbenchWindow();
                try {
                    window.run(true, true, new IRunnableWithProgress() {

                        public void run(IProgressMonitor monitor)
                                throws InvocationTargetException,
                                InterruptedException {
                            monitor.beginTask(title, IProgressMonitor.UNKNOWN);
                            try {
                                StringTokenizer tokenizer = new StringTokenizer(
                                        paths, ";");
                                while (tokenizer.hasMoreTokens()) {
                                    String path = tokenizer.nextToken();

                                    final DocumentNode document = NodeFactory
                                                .createDocumentNode(path);

                                    Display.getDefault().syncExec(
                                            new Runnable() {

                                                public void run() {
                                                    addDocument(document);
                                                }
                                            });
                                }
                            } catch (final Exception e) {
                                e.printStackTrace();
                                Display.getDefault().syncExec(new Runnable() {

                                    public void run() {
                                        String msg = "Unable to read the specified javadoc path \n"
                                                + e.getMessage();
                                        Shell parent = Display.getDefault()
                                                .getActiveShell();
                                        MessageDialog.openError(
                                                parent,
                                                "Error",
                                                msg);
                                    }
                                });
                            } finally {
                                monitor.done();
                            }
                        }
                    });
                } catch (InvocationTargetException e) {
                    // TODO 자동 생성된 catch 블록
                    e.printStackTrace();
                } catch (InterruptedException e) {
                    // TODO 자동 생성된 catch 블록
                    e.printStackTrace();
                }
            }
        }
    }

    public void addDocument(DocumentNode document) {
        document.sortChildren(new NodeComparator());
        documents.add(document);

        //TODO
        viewer.setInput(documents);
    }

    public void selectNodeWithURL(URL url) {
        for (DocumentNode document : documents) {
            ElementNode node = document.findChild(url);
            if (node != null) {
                viewer.setSelection(new StructuredSelection(node), true);
                break;
            }
        }
    }

    public void removeDocument(DocumentNode node) {
        documents.remove(node);
        //TODO
        viewer.setInput(documents);
    }

    /**
     * Passing the focus request to the viewer's control.
     */
    public void setFocus() {
        viewer.getControl().setFocus();
    }

    /**
     * @see org.eclipse.ui.IViewPart#init(org.eclipse.ui.IViewSite, org.eclipse.ui.IMemento)
     */
    @Override
    public void init(IViewSite site, IMemento memento) throws PartInitException {
        super.init(site, memento);

        this.memento = memento;
    }

    /**
     * @see org.eclipse.ui.IPersistable#saveState(org.eclipse.ui.IMemento)
     */
    @Override
    public void saveState(IMemento memento) {
        super.saveState(memento);

        StringBuffer buffer = new StringBuffer();

        for (DocumentNode document : documents) {
            String path = document.toExternalForm();
            buffer.append(path);
            buffer.append(";");
        }

        memento.putString(DOCUMENT_PATHS, buffer.toString());
    }
}